unit uInstrumental_Basico;

interface
uses SysUtils;
Type
Ponto = Class;

Vetor = Class
  Public
  x,y,z:real;
  constructor create(x1,y1,z1:real);
  procedure carrega(x1,y1,z1:real);overload;
  procedure carrega(p1,p2:ponto);overload;
  procedure copia(v:vetor);
  function eh_nulo:Boolean;
  function eh_multiplo_de(v:vetor):Boolean;
  function escalar(v:vetor):real;
  procedure mais(v:vetor);
  procedure menos(v:vetor);
  function norma:real;
  procedure simetrico;
  procedure vezes(v:vetor);overload;
  procedure vezes(k:real);overload;
  procedure vezesRMD(v:vetor);
  procedure normalizado;
  function clone:vetor;
  procedure vetor_terminado_no_ponto(p:ponto);
  function cprojecao(v:vetor):real;
  function projecao(v:vetor):vetor;
  procedure carrega_projecao(v:vetor);
end;

Ponto = Class
  Public
  x,y,z:real;
  xm,ym,zm:real;
  constructor create(x1,y1,z1:real);
  procedure carrega(x1,y1,z1:real);
  procedure carrega_mundial(x1,y1,z1:real);
  procedure mais(v:vetor);
  procedure menos(v:vetor);
  function sua_distancia_para(p:ponto):real;
  function clone:ponto;
  procedure copia(p:ponto);
end;

Cor = Class
  Public
  r,g,b:real;
  constructor create(r1,g1,b1:real);
  procedure carrega(r1,g1,b1:real);
  procedure mais(c:Cor);
  procedure vezes(c:Cor); overload;
  procedure vezes(k:real); overload;
  procedure truncada;
  procedure copia(c:Cor);
  procedure normalizada;
  function clone:cor;
end;

procedure produto(var c:Cor;k:real;O,I:Cor);
function obter_real(str:string; var erro:Boolean):real;
function obter_int(str:string; var erro:Boolean):integer;
procedure combinacao_baricentrica(Var Pt:Ponto; alpha:real;A:Ponto;beta:real;
                                B:Ponto;gamma:real;C:Ponto);

const Pequeno = 0.00000001;

var     FatorDilatacaoZ:real;

implementation

//------------------------------------------------------------------------
function obter_real(str:string; var erro:Boolean):real;
begin
  erro:=False;
  try
     result:=StrToFloat(str);
  except
     on E:EConvertError do erro:=True;
  end;
end;

//------------------------------------------------------------------------
function obter_int(str:string; var erro:Boolean):integer;
begin
  erro:=False;
  try
     result:=StrToInt(str);
  except
     on E:EConvertError do erro:=True;
  end;
end;

//------------------------------------------------------------------
constructor vetor.create(x1,y1,z1:real);
begin
  inherited create;
  x:=x1;
  y:=y1;
  z:=z1;
end;

procedure vetor.carrega(x1,y1,z1:real);
begin
   x:=x1;  y:=y1;   z:=z1;
end;

procedure vetor.copia(v:vetor);
begin
  x:=v.x; y:=v.y;  z:=v.z;
end;
function vetor.eh_nulo:boolean;
begin
  result:=(abs(x)<Pequeno)and(abs(y)<Pequeno)and(abs(z)<Pequeno);
end;

function vetor.escalar(v:vetor):real;
begin
  result:= x*v.x + y*v.y + z*v.z;
end;

procedure vetor.mais(v:vetor);
begin
  x:=x + v.x;
  y:=y + v.y;
  z:=z + v.z;
end;

procedure vetor.menos(v:vetor);
begin
  x:=x - v.x;
  y:=y - v.y;
  z:=z - v.z;
end;

function vetor.norma:real;
begin
  result:=sqrt(x*x + y*y + z*z);
end;

procedure vetor.simetrico;
begin
  x:= -x;
  y:= -y;
  z:= -z;
end;

procedure vetor.vezes(v:vetor);   //Prod. Vetorial Regra da Mo Esquerda
var xt,yt:real;
begin
  xt:= -y*v.z + z*v.y;
  yt:= -z*v.x + x*v.z;
  z:= -x*v.y + y*v.x;
  x:=xt; y:=yt;
end;

procedure vetor.vezesRMD(v:vetor);  //Prod. Vetorial Regra da Mo Direita
var xt,yt:real;
begin
  xt:= y*v.z - z*v.y;
  yt:= z*v.x - x*v.z;
  z:= x*v.y - y*v.x;
  x:=xt;  y:=yt;
end;

procedure vetor.normalizado;
var vnorma:real;
begin
   vnorma:= sqrt(x*x + y*y + z*z);
   if (vnorma > Pequeno) then begin
      x:= x/vnorma;
      y:= y/vnorma;
      z:= z/vnorma;
   end;
end;

function vetor.eh_multiplo_de(v:vetor):Boolean;
var a,b,c:real;
    xnulo,ynulo,znulo:boolean;
begin
  if v.eh_nulo then result:=False;
  a:=1; b:=1; c:=1;
  xnulo:=False; ynulo:=False; znulo:=False;
  if (abs(x)<Pequeno) then
     if (abs(v.x)>Pequeno) then result:=False
     else xnulo:=true
  else a:=v.x / x;

  if (abs(y)<Pequeno) then
     if (abs(v.y)>Pequeno) then result:=False
     else ynulo:=true
  else b:=v.y / y;

  if (abs(z)<Pequeno) then
     if (abs(v.z)>Pequeno) then result:=False
     else znulo:=true
  else c:=v.z / z;

  if xnulo then  begin
     if ynulo then result:=true
     else if (abs(b-c)<Pequeno) then result:=true
     else result:=false;
  end
  else begin
     if ynulo then begin
        if znulo then result:=true
        else if (abs(a-c)<Pequeno) then result:=true
        else result:=false;
     end
     else begin
        if (abs(a-b)>Pequeno) then result:=false
        else if znulo then result:=true
             else if (abs(a-c)<Pequeno) then result:=true
             else result:=false;
     end;
  end;
end;

function vetor.clone:vetor;
var vr:vetor;
begin
   vr:=vetor.create(0,0,0);
   vr.carrega(x,y,z);
   result:=vr;
end;

procedure vetor.carrega(p1,p2:ponto);
begin
  x:= p1.x - p2.x;
  y:= p1.y - p2.y;
  z:= p1.z - p2.z;
end;


procedure vetor.vetor_terminado_no_ponto(p:ponto);
begin
  x:=p.x;
  y:=p.y;
  z:=p.z;
end;

procedure vetor.vezes(k:real);
begin
   x:=x * k;
   y:=y * k;
   z:=z * k;
end;

function vetor.cprojecao(v:vetor):real;
begin
   if not v.eh_nulo then
      result:=escalar(v)/v.escalar(v)
   else
      result:=0;
end;

function vetor.projecao(v:vetor):vetor;
var w:vetor;
begin
   w:=vetor.create(0,0,0);
   if not v.eh_nulo then begin
      w.copia(v);
      w.vezes(escalar(v)/v.escalar(v));
   end;
   result:=w;
end;

procedure vetor.carrega_projecao(v:vetor);
var k:real;
begin
   if not v.eh_nulo then begin
      k:=escalar(v)/v.escalar(v);
      copia(v);
      vezes(k);
   end
   else
      carrega(0,0,0);
end;

//------------------------------------------------------------
constructor ponto.create(x1,y1,z1:real);
begin
   inherited create;
   xm:= x1;
   ym:= y1;
   zm:= z1;
end;

procedure ponto.carrega(x1,y1,z1:real);
begin
   x:=x1;  y:=y1;   z:=z1;
end;

procedure ponto.carrega_mundial(x1,y1,z1:real);
begin
   xm:=x1;  ym:=y1;   zm:=z1;
end;

procedure ponto.copia(p:ponto);
begin
  x:=p.x;
  y:=p.y;
  z:=p.z;
end;

procedure ponto.mais(v:vetor);
begin
  x:= x + v.x;
  y:= y + v.y;
  z:= z + v.z;
end;

procedure ponto.menos(v:vetor);
begin
  x:= x - v.x;
  y:= y - v.y;
  z:= z - v.z;
end;


function ponto.sua_distancia_para(p:ponto):real;
var xt,yt,zt:real;
begin
  xt:= x - p.x;
  yt:= y - p.y;
  zt:= z - p.z;
  result:= sqrt(xt*xt + yt*yt + zt*zt);
end;


function ponto.clone:ponto;
var pt:ponto;
begin
   pt:=ponto.create(pt.xm,pt.ym,pt.zm);
   pt.x:=x;
   pt.y:=y;
   pt.z:=z;
   result:=pt;
end;

 //------------------------------------------------------------
constructor cor.create(r1,g1,b1:real);
begin
   inherited create;
   r:= r1;
   g:= g1;
   b:= b1;
end;

procedure cor.carrega(r1,g1,b1:real);
begin
   r:=r1;  g:=g1;   b:=b1;
end;


procedure Cor.mais(c:cor);
begin
   r:= r + c.r;
   g:= g + c.g;
   b:= b + c.b;
end;

procedure Cor.vezes(c:cor);
begin
  r:= r * c.r;
  g:= g * c.g;
  b:= b * c.b;
end;

procedure Cor.vezes(k:real);
begin
  r:= r * k;
  g:= g * k;
  b:= b * k;
end;

procedure Cor.copia(C:Cor);
begin
  r:= C.r;
  g:= C.g;
  b:= C.b;
end;

procedure Cor.truncada;
begin
  if r > 255 then r:= 255;
  if g > 255 then g:= 255;
  if b > 255 then b:= 255;
end;

function Cor.clone:Cor;
var cr:Cor;
begin
  cr:=Cor.create(0,0,0);
  cr.r:=r;
  cr.g:=g;
  cr.b:=b;
  result:=cr;
end;

procedure Cor.normalizada;
begin
  r:=r/255;
  g:=g/255;
  b:=b/255;
end;

// Procedimentos e funces  gerais

procedure combinacao_baricentrica(Var Pt:Ponto; alpha:real;A:Ponto;beta:real;
                                B:Ponto;gamma:real;C:Ponto);

begin
   Pt.x := alpha * A.x + beta * B.x  + gamma * C.x;
   Pt.y := alpha * A.y + beta * B.y  + gamma * C.y;
   Pt.z := alpha * A.z + beta * B.z  + gamma * C.z;
end;

procedure produto(var c:Cor;k:real;O,I:Cor);
begin
   c.copia(O);
   c.vezes(k);
   c.r:=c.r * I.r;
   c.g:=c.g * I.g;
   c.b:=c.b * I.b;
end;


end.
